import React, { Component } from 'react'
import addons from '@storybook/addons'
import StorybookReduxAddon from './StorybookReduxAddon'

class ReduxPanel extends Component {
  constructor (props) {
    super(props)

    this.state = { store: null }
    this._addStore = this._addStore.bind(this)
    this._removeStore = this._removeStore.bind(this)
  }

  _addStore () {
    this.setState({ store: document.getElementById('storybook-preview-iframe').contentWindow.__STORE__ })
  }

  _removeStore () {
    document.getElementById('storybook-preview-iframe').contentWindow.__STORE__ = null
    this.setState({ store: null })
  }

  componentDidMount () {
    const { channel, api } = this.props

    channel.on('addons/redux/add_store', this._addStore)
    this._stopListeningOnStory = api.onStory(this._removeStore)
  }

  render () {
    const { store } = this.state
    const content = store ? (
      <StorybookReduxAddon store={store} />
    ) : null

    return content
  }

  componentWillUnmount () {
    const { channel } = this.props

    if (this._stopListeningOnStory) {
      this._stopListeningOnStory()
    }

    channel.removeListener('addons/redux/add_store', this._addStore)
  }
}

addons.register('addons/redux', (api) => (
  addons.addPanel('addons/redux/panel', {
    title: 'Redux',
    render: () => (
      <ReduxPanel
        channel={addons.getChannel()}
        api={api} />
    )
  })
))
